package brs.visitor;
import brs.*;
import java.util.*;

/**
 * Finds the next larger element with respect to a given input in a binary host tree 
 * that satisfies the Binary Search Tree Property.
 * @author DXN
 */
public class BSTFindNext implements IVisitor {
    private Comparator _order;
    /**
     * Used when the items in the tree are Comparable objects.
     */
    public BSTFindNext() {
        _order = new Comparator() {
            public int compare(Object x, Object y) {
                return ((Comparable)x).compareTo(y);
            }
        };
    }
    
    /**
     * Used when the items are ordered according to a given Comparator.
     * @param order a total ordering on the items stored in the tree.
     */
    public BSTFindNext(Comparator order) {
        _order = order;
    }
    
    /**
     * Returns the empty host tree itself.
     * @param host an empty BiTree
     * @param nu not used
     * @return BiTree the empty host tree.
     */
    public Object emptyCase(BiTree host, Object... nu) {
        return host;
    }
    
    /**
     * Returns the BiTree whose root is the next largest element with respect to 
     * the input.  The input need not be in the BiTree.
     * If there is no next largest element, return an empty BiTree.
     * @param host non empty and satisfies BSTP.
     * @param input[0] object to be compared in order to find the next largest element.
     * @return BiTree whose root is the next largest element with respect to inp[0].
     */
    public Object nonEmptyCase(BiTree host, final Object... input) {
        if (_order.compare(input[0], host.getRootDat()) < 0) {  // input < root, look left
            return host.getLeftSubTree().execute(new IVisitor() {
                // inp[1] is an ancestor tree.
                public Object emptyCase(BiTree h, Object... inp) {
                    return inp[0];  // 
                }
                
                public Object nonEmptyCase(BiTree h, Object... inp) {
                    if (_order.compare(input[0], h.getRootDat()) < 0) {
                        return h.getLeftSubTree().execute(this, h);
                    }
                    else {  // input is greater or equal to root, look right.
                        return h.getRightSubTree().execute(this, inp);
                    }
                }
            }, host);
        }
        else { // input[0] >= root, look right
            return host.getRightSubTree().execute(this, input[0]);
        }
    }
}

